function [sgout]=sg( tau, vclose, fc, cyc_dig, Pn, snrdb, law, harm)
% function [sgout]=sg( tau, vclose, fc, cyc_dig, Pn, snrdb, law, harm)
% 
% linear maximal length sequence signal generator
%
% tau=0.0;         % delay in seconds
% vclose=0.0       % closing speed in m/s
% fc=75;           % carrier frequency
% cyc_dig=2;       % cycles per digit ( or Q)
% Pn=10;           % periods per transmission
% snrdb=0.0;       % input signal to noise ratio
% law = [ 1 0 0 0 0 0 1 1 0 1 1];  % 2033 oct
% harm = 1;        % harmonic default=1, AST 84 Hz=3;
% states associated with generator polynomial
% sam_cyc=4;       % samples per cycle
% bits=12;         % A/D bits
% bits=16;         % A/D bits
% foverload=25;    % ratio of A/D full scale range to signal rms
% foverload=64;    % TGB standard operating point

if nargin==0
  tau=0.0;  vclose=0.0;
  fc=75; cyc_dig=2; Pn=10; snrdb=0.0;
  law = [ 1 0 0 0 0 0 1 1 0 1 1];
  harm=1;
elseif nargin==1
  vclose=0.0;
  fc=75; cyc_dig=2; Pn=10; snrdb=0.0;
  law = [ 1 0 0 0 0 0 1 1 0 1 1];
  harm=1;
elseif nargin==2
  fc=75; cyc_dig=2; Pn=10; snrdb=0.0;
  law = [ 1 0 0 0 0 0 1 1 0 1 1];
  harm=1;
elseif nargin==3
  cyc_dig=2; Pn=10; snrdb=0.0;
  law = [ 1 0 0 0 0 0 1 1 0 1 1];
  harm=1;
elseif nargin==4
  Pn=10; snrdb=0.0;
  law = [ 1 0 0 0 0 0 1 1 0 1 1];
  harm=1;
elseif nargin==5
  snrdb=0.0;
  law = [ 1 0 0 0 0 0 1 1 0 1 1];
  harm=1;
elseif nargin==6
  law = [ 1 0 1 1];                % 13 oct
  law = [ 1 0 0 1 1 ];             % 23 oct
  law = [ 1 0 0 1 0 1];            % 45 oct
  law = [ 1 0 0 0 0 1 1];          % 103 oct
  law = [ 1 0 0 0 0 0 1 1];        % 203 oct
  law = [ 1 0 0 0 1 1 1 0 1];      % 435 oct
  law = [ 1 0 0 0 0 1 0 0 0 1];    % 1021 oct
  law = [ 1 0 0 0 0 0 1 1 0 1 1];  % 2033 oct
  harm=1;
elseif nargin==7
  harm=1;
end

sam_cyc=4;       % samples per cycle
bits=12;         % A/D bits
%bits=16;         % A/D bits
foverload=25;     % ratio of A/D full scale range to signal rms
foverload=64;     % TGB standard operating point

law=law(:);
nr=length(law)-1;             % length of sequence register
L = 2.^nr - 1;                % digits per period
thetamod=-atan(sqrt(L));      % modulation angle
thetamod=harm*thetamod;
sam_dig=sam_cyc*cyc_dig;      % samples per digit
fs=sam_cyc*fc;                % sampling frequency
N=sam_cyc*cyc_dig*L;          % samples per period
T=N/fs;                       % period length
vopen=-vclose;                % opening velocity
cspeed=2913;                  % KM value
cspeed=1484;
alpha=1-vopen/cspeed;         % doppler compression factor

% ssrg (simple shift register generator)
G=[ law(2:nr+1)'; [eye(nr-1) zeros(nr-1,1)] ];
Q=zeros(nr,L); Q(1,1)=1;
for il=1:L-1
   Q(:,il+1) = rem(G*Q(:,il),2);
end
Q=[Q(:,L-nr+2:L), Q(:,1:L-nr+1)]; %delay to match KM
Qp=2.^[0:nr-1]*Q;

% msrg (modular shift register generator)
G=[ [zeros(1,nr-1); eye(nr-1)] law(1:nr) ];
P=zeros(nr,L); P(1,1) = 1;
for il=1:L-1
   P(:,il+1)=rem(G*P(:,il),2);
end
Pp=2.^[0:nr-1]*P;

% modulation

j=sqrt(-1);
dg=(1-2*Q(1,:));
%u=1/tan(thetamod); v=j;
%dmod = u + v*dg;

%transmit signal

n=1:1:Pn*N; n=n-1;
Ns=length(n);
ntau=alpha*n-tau*fs;
mi=floor( ntau/sam_dig);
mi=rem( mi, L);
while mi(1)<0
  mi=mi+L;
  mi=rem( mi, L);
end
mi=mi+1;
sig=cos( 2*pi*fc*ntau/fs + thetamod*dg(mi));
%sig=exp(j*( 2*pi*fc*ntau/fs + thetamod*dg(mi)));

%For complex demodulates
%x=normrnd(0,5,100000,1);
%y=normrnd(0,5,100000,1);
%z=x.*x+y.*y;
%log(2)
%median(z)/std(z)

%For samples
%x=normrnd(0,7,100000,1);
%z=x.*x;
%chi2inv(0.5, 1)
%md2rms=sqrt(2)*erfinv(0.5); md2rms*md2rms
%median(z)/var(x)

% add white noise
snr=10.0.^(snrdb/20.0);
sig=snr*sig/std(sig)+randn(1,Ns);

%hydrophone preamp response
GM=10^(-168/20); %%measured hydrophone sensitivity

R5=17400;
C4=390e-12;
R4=499;
C3=33e-6;

fchyd=75;
Z1=R5./(1+j*2*pi*fchyd*R5*C4);
Z2=(1+j*2*pi*fchyd*R4*C3)./(j*2*pi*fchyd*C3);
G0=abs((Z1+Z2)./Z2);

[ num, den]=zp2tf([ 0.0], [ -1/R5/C4 -1/R4/C3], -GM/R4/C4/G0);
[ firb, fira]=bilinear( num, den, fs);
clear num den G0 GM R5 C4 R4 C3 fchyd Z1 Z2A
sig=filter( firb, fira, sig);

%avatoc frontend low-pass filter (anti-strum)

fcut=7.2;
rc=1/fcut/2/pi;
[ firb, fira]=zp2tf([ 0.0], [ -1/rc], 1.0);
[ firb, fira]=bilinear( firb, fira, 300);
%freqz( firb, fira, [5:0.1:30], 300);
%pause
%gd=grpdelay( firb, fira, [5:0.1:30], 300);
%plot( [5:0.1:30], 1000*gd/300)
%grid on
% pause
sig=filter( firb, fira, sig);

%avatoc frontend hi-pass filter (anti-alias)

[ firb fira] = butter( 6, 100/150); 
%freqz( firb, fira, [5:1:100], 300);
%pause
%gd=grpdelay( firb, fira, [60:0.5:90], 300);
%plot( [60:0.5:90], 1000*gd/300)
%grid
%pause
sig=filter( firb, fira, sig);

%scale correctly for A/D
gain=2^(bits-1)/foverload;
sig=gain*sig/std(sig);

%clip and quantize
ind=find( sig>(2^(bits-1)));
sig(ind)=(2^(bits-1))*ones(size(ind));
ind=find( sig<-(2^(bits-1)));
sig(ind)=-(2^(bits-1))*ones(size(ind));
sig=round( sig);

%write out data

if nargout==0
  time=clock;
  yr=time(1);
  yd=dt2yd( time(1)-1, 12, 31);
  yd=dt2yd( time(1), time(2), time(3))-yd;
  hr=time(4); mn=time(5); sc=round(time(6));
  start=sprintf( '%4d:%3d:%2d:%2d:%2d', yr, yd, hr, mn, sc);

  fid=fopen( 'proto.sam', 'w', 'ieee-le');
  fprintf( fid, '*HEADER*\n');
  fprintf( fid, 'BEGIN_PROGRAM: sg.m\n');
  fprintf( fid, '  site: mad\n');
  fprintf( fid, '  ad_id: matlab\n');
  fprintf( fid, '  data_description:');
  fprintf( fid, '  binary two_bytes little_endian integer real\n');
  fprintf( fid, '  start_time: %s\n', start);
  fprintf( fid, '  sample_rate: %.0f\n', fs);
  fprintf( fid, '  cycles_per_digit: %.0f\n', cyc_dig);
  fprintf( fid, '  samples_per_cycle: %.0f\n', sam_cyc);
  fprintf( fid, '  sequence_repetititions: %.0f\n', Pn);
  fprintf( fid, '  coordinate: 1 1 phone\n');
  fprintf( fid, '  coord_num: 0\n');
  fprintf( fid, '  coordinate: 2 %.0f sample\n', Ns);
  fprintf( fid, 'END_PROGRAM: sg.m\n');
  fprintf( fid, '*DATA*\n');
  fwrite( fid, sig, 'short');
  fclose(fid);
else
  sgout=sig;
end

return

